float filteredpulse (float edge0, edge1, x, dx)
{
    float x0 = x - dx/2;
    float x1 = x0 + dx;
    return max (0, (min(x1,edge1)-max(x0,edge0)) / dx);
}


void basicbrick (float x, y;
                uniform float tilewidth, tileheight;
                uniform float rowstagger, rowstaggervary;
                uniform float jaggedfreq, jaggedamp;
                output float column, row;
                output float xtile, ytile;
    )
{
    point PP;
    float scoord = x, tcoord = y;

    if (jaggedamp != 0.0) {
        /* Make the shapes of the bricks vary just a bit */
        PP = point noise (x*jaggedfreq/tilewidth, y*jaggedfreq/tileheight);
        scoord += jaggedamp * xcomp (PP);
        tcoord += jaggedamp * ycomp (PP);
    }

    xtile = scoord / tilewidth;
    ytile = tcoord / tileheight;
    row = floor (ytile);   /* which brick row? */

    /* Shift the columns randomly by row */
    xtile += mod (rowstagger * row, 1);
    xtile += rowstaggervary * (noise (row+0.5) - 0.5);

    column = floor (xtile);
    xtile -= column;
    ytile -= row;
}

surface
brick ( float Ka = 1, Kd = 1;
        color brickcolor = color "rgb" (.6,.1,.1);
        color mortarcolor = color "rgb" (.6,.6,.6);
        float raggedamp = 0.04, raggedfreq = 12;
        float jagged = 0.006, brickvary = 0.3;
        float brickwidth = .28, brickheight = .07;
        float mortarthickness = .014;
        float rowvary = .5;
        float pitting = 0.01;
        float pockfrequency = 10, groovedepth = 0.01;
    )
{

    color bcolor, Ct;
    normal Nf = normalize(N);
    float sbrick, tbrick, w, h;
    float ss, tt;
    float swidth, twidth;
    uniform float BMWIDTH = (brickwidth+mortarthickness);
    uniform float BMHEIGHT = (brickheight+mortarthickness);
    uniform float MWF = (mortarthickness*0.5/BMWIDTH);
    uniform float MHF = (mortarthickness*0.5/BMHEIGHT);
    float whichbrick;

    swidth = 0.00001;
    twidth = 0.00001;
    
    basicbrick (s, t, BMWIDTH, BMHEIGHT, 0.5, 0.2, 1, jagged,
                sbrick, tbrick, ss, tt);

    /* Make the edges ragged, but different for each brick */
    whichbrick = 103*sbrick + tbrick;
    ss += raggedamp * (2 * (float noise( (s+tbrick*5.15)*raggedfreq , 
                                (t+sbrick*23.8)*raggedfreq )) - 1) ;
    tt += raggedamp * (2 * (float noise( (s+tbrick*11.4)*raggedfreq , 
                                (t+sbrick*7.2)*raggedfreq )) - 1) ;
    ss += raggedamp/2 * (2 * (float noise( (s+tbrick*5.15)*raggedfreq*2 , 
                                  (t+sbrick*23.8)*raggedfreq*2 )) - 1) ;
    tt += raggedamp/2 * (2 * (float noise( (s+tbrick*11.4)*raggedfreq*2 , 
                                  (t+sbrick*7.2)*raggedfreq*2 )) - 1) ;

    /* Choose a color for the surface */
    if (swidth >= 1)
        w = 1 - 2*MWF;
    else w = clamp (filteredpulse (MWF, 1-MWF, ss, swidth), max(1-MWF/swidth,0), 1);
    if (twidth >= 1)
        h = 1 - 2*MHF;
    else h = clamp (filteredpulse (MHF, 1-MHF, tt, twidth), max(1-MHF/twidth,0), 1);
    
    /* Choose a brick color that varies from brick to brick */
    bcolor = brickcolor * (1 + (brickvary * (2 * (float noise( whichbrick+0.5 )) - 1) ));

    Ct = mix (mortarcolor, bcolor, w*h);
    Oi = 1;
    Ci = Ct * (Ka * ambient() + Kd*diffuse(Nf));
}
