/*
  VKFcloud
  Fake cloud shader for POVMan by Vahur Krouverk
  contact : vahur@aetec.ee
  created : March, 2001
  
*/


float fBm (point p; float filtwidth;
           uniform float octaves, lacunarity, gain)
{
    uniform float amp = 1;
    varying point pp = p;
    varying float sum = 0, fw = filtwidth;
    uniform float i;

    for (i = 0;  i < octaves;  i += 1) {
        sum += amp * ((2*noise(pp)-1)*(1-smoothstep(0.2, 0.75, fw)));
	amp *= gain;  pp *= lacunarity;  fw *= lacunarity;
    }
    return sum;
}




surface VKFcloud 
(
    float Kd = 0.5;  // diffuse
    float Ka = 0.4;  // ambient
    string shadingspace = "shader";
    // fBm values
    float freq = 4, octaves = 6, lacunarity = 2, gain = 0.5; 
    // light source position
    vector lsource = (0,1,0);
    // iteration count
    float stepcount = 20;

    // filter size for fBm function. Just hard-coded, as surface derivatives
    // are not implemented. Try various values for best results!
    float dPshad = 1e-4;
)
{

    // light source in shader space
    point Lsrc= transform(shadingspace, lsource)*freq;

    point Pshad = freq * transform (shadingspace, P);
    point sampleP = Pshad;
    // initial noise value
    float val = fBm(sampleP, dPshad, octaves, lacunarity, gain);
    float val1 = val;
    float count = 0;
    // vector toward light source
    vector lvect = normalize(Lsrc-Pshad);

    while(count < stepcount){
        count += 1;
        if (val1 < 0){
            break;
        }
        // march toward light source...
        sampleP = Pshad + count*lvect/stepcount;
        // and calculate total noise value
        val1 += (fBm(sampleP, dPshad, octaves, lacunarity, gain));
    }

    if (count == stepcount){
        // no clouds, make surface transparent
        Oi = 0;
        Ci = 1;
    }
    else{
        // modify fBm with iteration steps: the more steps required, the more
        // current point "is" toward light source and the better lighted.
//printf("%f, %f\n", val1, count);
        val1 = ((val1+1)/2)*(0.5+(count/(stepcount)));
        // fudge result a little bit smoother. Experiment with various values
        // to get best results
        val1 = pow(val1, 0.8);
        val1 = smoothstep(-0.2, 0.9, val1);
        Ci = val1*Kd*Cs + Ka*ambient();
        Oi= 1;
    }      

}
