vector vsnoise(vector x)
{
    return 2*vector(noise(x))-1;
}

vector vfBm(point p; uniform float octaves, lacunarity, gain)
{
    uniform float amp = 1;
    uniform point pp = p;
    varying vector sum = 0;
    uniform float i;
    for (i=0; i<octaves; i+= 1){
        sum += amp*vsnoise(pp);
        amp *= gain;
        pp *= lacunarity;
    }
    return sum;
}

void voronoi_f1_3d(point P; float jitter; output float f1; output point pos1;)
{
    point thiscell = point(floor(xcomp(P))+0.5, floor(ycomp(P))+0.5, floor(zcomp(P))+0.5);
    f1 = 1000;
    uniform float i, j, k;
    for (i=-1; i<=1; i += 1){
        for (j=-1; j<=1; j += 1){
            for (k=-1; k<=1; k += 1){
                point testcell = thiscell+vector(i,j,k);
                point pos = testcell + jitter*(vector cellnoise(testcell)-0.5);
                vector offset = pos - P;
                float dist = offset.offset;
                if (dist < f1){
                    f1 = dist;
                    pos1 = pos;
                }
            }
        }
    }
    f1 = sqrt(f1);
}


void voronoi_f1f2_3d(point P; float jitter; 
        output float f1; output point pos1;
        output float f2; output point pos2;)
{
    point thiscell = point(floor(xcomp(P))+0.5, floor(ycomp(P))+0.5, floor(zcomp(P))+0.5);
    f1 = f2 = 1000;
    uniform float i, j, k;
    for (i=-1; i<=1; i += 1){
        for (j=-1; j<=1; j += 1){
            for (k=-1; k<=1; k += 1){
                point testcell = thiscell+vector(i,j,k);
                point pos = testcell + jitter*(vector cellnoise(testcell)-0.5);
                vector offset = pos - P;
                float dist = offset.offset;
                if (dist < f1){
                    f2 = f1;
                    pos2 = pos1;
                    f1 = dist;
                    pos1 = pos;
                }
                else if (dist < f2){
                    f2 = dist;
                    pos2 = pos;
                }
            }
        }
    }
    f1 = sqrt(f1);
    f2 = sqrt(f2);
}


surface voronoi_pattern(float scale=0.05, fracScale = 0.15)
{
    float fracLac = 2;
    point PP = P + fracScale*vfBm(2*P, 4, fracLac, 1/fracLac);
    
    float f1, f2;
    point pos1, pos2;
    
    voronoi_f1f2_3d(PP, 1, f1, pos1, f2, pos2);
    float scalefactor = distance(pos1, pos2)/(distance(pos1, P) + distance(P, pos2));
    Ci = step(scale*scalefactor, f2-f1);
}



